/* MPEG Audio info
 * By Fredrik Noring, April 1996
 */

#include <stdio.h>

#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define         MPG_MD_MONO             3

double frequencies[4] = {44.1, 48, 32, 0}; 
int bitrates[3][15] = {
  {0,32,64,96,128,160,192,224,256,288,320,352,384,416,448},
  {0,32,48,56,64,80,96,112,128,160,192,224,256,320,384},
  {0,32,40,48,56,64,80,96,112,128,160,192,224,256,320}
};

int main(int argc, char *argv[])
{
  FILE *fp;
  double frequency;
  int info, bpc, stereo, table;
  int layer, crc, bitrate, copyright, original, mode;
  
  printf("MPEG2 Audio Info\n");

  if (argc == 1)
    fp = stdin;
  else
    if ((fp = fopen(argv[1], "rb")) == NULL) {
      fprintf(stderr, "%s: Couldn't open file \"%s\"\n", argv[0], argv[1]);
      return -1;
    }
  fread(&info, 4, 1, fp);
  fclose(fp);

  printf("info: %x\n", info);

  if (((info >> 20) & 0xfff) == 0xfff)
    printf("sync ok\n");
  else
    printf("sync NOT found\n");
  printf("\n");
  
  layer = 4 - ((info >> 17) & 0x3);
  printf("layer: %d\n", layer);

  mode = ((info >> 6) & 0x3);
  switch (mode) {
  case MPG_MD_STEREO:
    printf("stereo\n");
    break;
  case MPG_MD_JOINT_STEREO:
    printf("joint stereo\n");
    break;
  case MPG_MD_DUAL_CHANNEL:
    printf("dual channel\n");
    break;
  case MPG_MD_MONO:
    printf("mono\n");
    break;
  }
  if (mode ==  MPG_MD_MONO)
    stereo = 1;
  else
    stereo = 2;
  
  crc = !((info >> 16) & 0x1);
  if (crc)
    printf("error protected\n");
  else
    printf("no crc checksums\n");

  bitrate = bitrates[layer-1][((info >> 12) & 0xf)];
  bpc = bitrate / stereo;
  printf("bitrate: %dkbit/sec (bpc: %dkbit/sec)\n", bitrate, bpc);

  frequency = frequencies[((info >> 10) & 0x3)];
  printf("frequency: %fkHz\n", frequency);

  if ((frequency == 48 && bpc >= 56) ||
      (bpc >= 56 && bpc <= 80)) table = 0;
  else if (frequency != 48 && bpc >= 96) table = 1;
  else if (frequency != 32 && bpc <= 48) table = 2;
  else table = 3;
  printf("table: alloc_%d\n", table);

  original = ((info >> 2) & 0x1);
  printf("original: %d\n", original);
  copyright = ((info >> 3) & 0x1);
  printf("copyright: %d\n", copyright);
  
  return 0;
}
